package com.persagy.dmp.middleware.controller;


import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.dmp.common.constant.CommonConstant;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.vo.WdConstructDescription;
import com.persagy.dmp.middleware.service.IConstructDescriptionService;
import com.persagy.dmp.middleware.service.IEquipService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import com.persagy.dmp.middleware.utils.SortUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;
import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName ObjectInstanceController
 * @Description: 设备设施模块
 * @Author linhuili
 * @Date 2021/9/10 11:17
 * @Version V1.0
 **/
@RestController
@RequestMapping("equip")
public class EquipController {

    @Autowired
    private IEquipService equipService;

    @Autowired
    private IConstructDescriptionService constructDescriptionService;

    /**
     * 查询/统计设备设施下的设备/阀门
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public CommonResult count(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getBuildId())){
            throw new Exception("建筑ID不能为空");
        }
        //查询设备空间信息
        List<JSONObject> content = equipService.query(context, request);
        //对相同系统编码，相同类型名称 bimTypeId不一样的对象数据，类型合并
        content = EntityTransferUtil.combineData(content);
        //按首字母，对象数量为0放后的规则进行排序
        content = SortUtil.sortByCharacter(content,CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(content);
        response.setCount(content == null ? 0 : Long.valueOf(content.size()));
        return  response;
    }

    /**
     *  查询设备设施-暖通系统-全局-3D原理图相关的设备/阀门
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/havcSystem/query")
    public CommonResult queryHavcSystem(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(StringUtils.isEmpty(request.getSystemCode())){
            throw new Exception("系统编码不能为空");
        }
        //查询冷却塔设备类下的设备，制冷机房空间下的设备/阀门信息
        List<JSONObject> content = equipService.queryHavcSystem(context,request);

        //类型名称相同的合并处理
        content = EntityTransferUtil.combineData(content);
        //按首字母，对象数量为0放后的规则进行排序
        content = SortUtil.sortByCharacter(content,CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(content);
        response.setCount(content == null ? 0 : Long.valueOf(content.size()));
        return  response;
    }

    /**
     * 设备设施-暖通系统-分集水器原理图查询
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/subCatchment/query")
    public CommonResult querysubCatchment(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                        @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        //查询分水器、集水器类型下的对象信息
        List<JSONObject> content =  equipService.querysubCatchment(context,request);

        //类型名称相同的合并处理
        content = EntityTransferUtil.combineData(content);
        //按首字母，对象数量为0放后的规则进行排序
        content = SortUtil.sortByCharacter(content,CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(content);
        response.setCount(content == null ? 0 : Long.valueOf(content.size()));
        return  response;
    }

    /**
     * 查询土建装饰说明
     * @param projectId
     * @param vo
     * @return
     * @throws Exception
     */
    @PostMapping("/construction/query")
    public CommonResult queryConstructionDesc(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                          @RequestBody WdConstructDescription vo) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(StringUtils.isEmpty(vo.getFloorId())){
            throw new Exception("楼层id不能为空");
        }
        //添加默认条件
        List<WdConstructDescription> vos = new ArrayList<>();
        vos.add(vo);
        createDefaultValue(vos);
        //查询土建装饰说明
        CommonResult response = constructDescriptionService.query(context,vos.get(0));
        return  response;
    }


    /**
     * 添加土建装饰说明
     * @param projectId
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/construction/insert")
    public CommonResult insertConstructionDesc(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                              @RequestBody List<WdConstructDescription> vos) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(CollUtil.isEmpty(vos)){
            throw new Exception("待添加数据不能为空");
        }
        //添加默认条件
        createDefaultValue(vos);
        //添加土建装饰说明
        CommonResult response = constructDescriptionService.insert(context,vos);
        return  response;
    }

    /**
     * 修改土建装饰说明
     * @param projectId
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/construction/update")
    public CommonResult updateConstructionDesc(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                               @RequestBody List<WdConstructDescription> vos) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(CollUtil.isEmpty(vos)){
            throw new Exception("待修改数据不能为空");
        }
        //添加默认条件
        createDefaultValue(vos);
        //删除土建装饰说明
        CommonResult response = constructDescriptionService.update(context,vos);
        return  response;
    }

    /**
     * 删除土建装饰说明
     * @param projectId
     * @param vos
     * @return
     * @throws Exception
     */
    @PostMapping("/construction/delete")
    public CommonResult deleteConstructionDesc(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                               @RequestBody List<String> vos) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(CollUtil.isEmpty(vos)){
            throw new Exception("id不能为空");
        }
        //删除土建装饰说明
        CommonResult response = constructDescriptionService.delete(context,vos);
        return  response;
    }


    /**
     * 查询楼层地板
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/floorBoard/query")
    public CommonResult queryFloorBoard(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        if(StringUtils.isEmpty(request.getFloorId())){
            throw new Exception("楼层id不能为空");
        }
        //查询楼层地板
        List<JSONObject> content = equipService.queryFloorBoard(context,request);

        //类型名称相同的合并处理
        content = EntityTransferUtil.combineData(content);
        //按首字母，对象数量为0放后的规则进行排序
        content = SortUtil.sortByCharacter(content,CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(content);
        response.setCount(content == null ? 0 : Long.valueOf(content.size()));
        return  response;
    }


    /**
     * 创建默认条件
     * @param voList
     */
    private void createDefaultValue(List<WdConstructDescription> voList){
        String groupCode = AppContext.getContext().getGroupCode();
        if(StringUtils.isEmpty(groupCode)){
            //默认集团
            groupCode = CommonConstant.DEFAULT_ID;
        }
        String projectId = AppContext.getContext().getProjectId();
        if(StringUtils.isEmpty(projectId)){
            //默认项目
            projectId = CommonConstant.DEFAULT_ID;
        }
        for (WdConstructDescription vo : voList) {
            vo.setGroupCode(groupCode);
            vo.setProjectId(projectId);
        }
    }



}
