package com.persagy.dmp.middleware.controller;


import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.service.ISpaceService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import com.persagy.dmp.middleware.utils.SortUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.QueryParam;
import java.util.List;

/**
 * @ClassName ObjectInstanceController
 * @Description: 空间模块
 * @Author linhuili
 * @Date 2021/9/10 11:17
 * @Version V1.0
 **/
@RestController
@RequestMapping("space")
public class SpaceController {

    @Autowired
    private ISpaceService spaceService;

    @Autowired
    private IBaseService baseService;

    /**
     * 查询空间详情
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/query")
    public CommonResult query(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getId())){
            throw new Exception("空间ID不能为空");
        }
        //封装中台查询条件
        QueryCriteria queryCriteria = commbineDmpCriteria(request);
        //查询空间信息
        CommonResult response = baseService.doQuery(context,queryCriteria, JSONObject.class);
        return  response;
    }

    /**
     * 查询空间内的设备/阀门
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/rel/query")
    public CommonResult relQuery(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                              @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        // 非空校验
        if(StringUtils.isEmpty(request.getSpaceId())){
            throw new Exception("空间ID不能为空");
        }
        if(StringUtils.isEmpty(request.getSystemCode())){
            throw new Exception("系统编码不能为空");
        }
        //查询空间内的设备/阀门
        List<JSONObject> content = spaceService.relQuery(context,request);

        //类型名称相同的合并处理
        content = EntityTransferUtil.combineData(content);
        //按首字母，对象数量为0放后的规则进行排序
        content = SortUtil.sortByCharacter(content, CommonConsant.OBJ_TYPE, CommonConsant.CLASS_NAME);
        //设置返回值
        CommonResult<List<JSONObject>> response = CommonResult.success(content);
        response.setCount(content == null ? 0 : Long.valueOf(content.size()));
        return  response;
    }

    /**
     * 查询防火分区下的空间
     * @param projectId
     * @param request
     * @return
     * @throws Exception
     */
    @PostMapping("/fireZone/query")
    public CommonResult queryFireZone(@QueryParam("groupCode") String groupCode, @QueryParam("projectId") String projectId,
                                 @RequestBody CommonRequestDTO request) throws Exception {
        // 组装上下文条件
        InstanceUrlParam context = new InstanceUrlParam(groupCode, null, projectId, null);
        //查询防火分区下的空间
        return spaceService.queryFireZone(context,request);
    }

    /**
     * 封装中台查询条件
     * @param requestDTO
     * @return
     */
    private QueryCriteria commbineDmpCriteria(CommonRequestDTO requestDTO){
        ObjectNode jsonNodes = JsonNodeFactory.instance.objectNode();
        jsonNodes.put("id",requestDTO.getId());
        QueryCriteria criteria = new QueryCriteria();
        criteria.setCriteria(jsonNodes);
        return criteria;
    }

    
}
