package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.constant.enums.ClassCodeEnum;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.constant.enums.WdDefTypeEnum;
import com.persagy.dmp.middleware.constant.enums.WdModeCodeEnum;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.vo.WdDefineTypeRel;
import com.persagy.dmp.middleware.service.*;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName FloorServiceImpl
 * @Description: 楼层
 * @Author linhuili
 * @Date 2021/9/9 10:24
 * @Version V1.0
 **/
@Service
public class FloorServiceImpl implements IFloorService {

    @Autowired
    private IBaseService baseService;
    
    @Autowired
    private ISpaceService spaceService;

    @Autowired
    private IEquipService equipService;

    @Autowired
    private IWdDefineTypeRelService iWdDefineTypeRelService;

    /**
     * 统计楼层下各信息
     * @param context
     * @param request
     * @return
     */
    @Override
    public CommonResult count(InstanceUrlParam context, CommonRequestDTO request) throws Exception {
        ArrayNode result = JsonNodeFactory.instance.arrayNode();

        //查询楼层信息
        List<JSONObject> floorList = queryFloorInfo(context, request);
        if(CollUtil.isEmpty(floorList)){
            throw new Exception("楼层不存在");
        }
        JSONObject floor = floorList.get(0);
        ObjectNode content = JsonNodeFactory.instance.objectNode();
        //统计楼层面积
        content.put("buildArea",floor.getDouble(CommonConsant.FLOOR_AREA));
        //统计租赁面积
        content.put("leaseArea",floor.getDouble(CommonConsant.FLOOR_LEASE_AREA));
        //统计该楼层各商铺数量
        Long storeNum = countStoreNum(context, request);
        content.put("storeNum",storeNum == null ? 0 : storeNum);

        //统计该楼层机动车位数量
        Long parkingCarNum = countParkingCar(context, request);
        content.put("countParkingCar",parkingCarNum == null ? 0 : parkingCarNum);

        result.add(content);
        return CommonResult.success(content);
    }

    /**
     * 查询广联达楼层下的设备，空间，阀门
     * @param context
     * @param request
     * @return
     */
    @Override
    public List<JSONObject> query(InstanceUrlParam context, CommonRequestDTO request) {
        //查询楼层下的设备，空间，阀门
        List<JSONObject> result = new ArrayList<>();
        //查询楼层下的空间信息
        List<JSONObject> spaceList = queryFloorSpaceList(context, request);
        if(CollUtil.isNotEmpty(spaceList)){
            result.addAll(spaceList);
        }
        //查询楼层下的设备
        List<JSONObject> equipList = queryFloorEquipList(context, request);
        if(CollUtil.isNotEmpty(equipList)){
            result.addAll(equipList);
        }
        //查询楼层下的阀门
        List<JSONObject> materiaList = queryFloorMateriaList(context, request);
        if(CollUtil.isNotEmpty(materiaList)){
            result.addAll(materiaList);
        }
        return result;
    }

    /**
     * 获取楼层下的空间
     * @param context
     * @param request
     */
    private List<JSONObject>  queryFloorSpaceList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达空间类型
        WdDefineTypeRel defineTypeRel = new WdDefineTypeRel();
        defineTypeRel.setObjType(WdDefTypeEnum.SPACE.getValue());
        defineTypeRel.setWdModelCode(WdModeCodeEnum.LC.getValue());
        List<WdDefineTypeRel> defineTypeRels = iWdDefineTypeRelService.query(context,defineTypeRel);

        //过滤不需要的类型，只需要楼层功能相关的空间
        //List<WdDefineTypeRel> floorDefineTypeRels = defineTypeRels.stream().filter(k->"LC".equals(k.getWdModelCode())).collect(Collectors.toList());

        //根据广联达空间类型查询空间信息
        List<JSONObject> spaceList = spaceService.querySpaceByGldTypeRel(context, defineTypeRels, request);
        return spaceList;
    }


    /**
     * 获取楼层下的设备
     * @param context
     * @param request
     */
    private List<JSONObject>  queryFloorEquipList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达设备类型
        WdDefineTypeRel defineTypeRel = new WdDefineTypeRel();
        defineTypeRel.setObjType(WdDefTypeEnum.EQUIPMENT.getValue());
        List<WdDefineTypeRel> defineTypeRels = iWdDefineTypeRelService.query(context,defineTypeRel);

        //根据广联达设备类型查询设备信息
        List<JSONObject> equipList = equipService.queryEquipByGldTypeRel(context, defineTypeRels, request);
        return equipList;
    }

    /**
     * 获取楼层下的设备
     * @param context
     * @param request
     */
    public List<JSONObject>  queryFloorMateriaList(InstanceUrlParam context, CommonRequestDTO request){
        //查询广联达设备类型
        WdDefineTypeRel defineTypeRel = new WdDefineTypeRel();
        defineTypeRel.setObjType(WdDefTypeEnum.MATERIAL.getValue());
        List<WdDefineTypeRel> defineTypeRels = iWdDefineTypeRelService.query(context,defineTypeRel);

        //根据广联达设备类型查询阀门信息
        List<JSONObject> materiaList = equipService.queryMateriaByGldTypeRel(context, defineTypeRels, request);
        return materiaList;
    }

    /**
     * 查询楼层信息
     * @param context
     * @param requestDTO
     * @return
     */
    private List<JSONObject> queryFloorInfo(InstanceUrlParam context, CommonRequestDTO requestDTO){
        ObjectNode jsonNodes = JsonNodeFactory.instance.objectNode();
        jsonNodes.put("id",requestDTO.getFloorId());
        QueryCriteria criteria = new QueryCriteria();
        criteria.setCriteria(jsonNodes);

        CommonResult response = baseService.doQuery(context, criteria, JSONObject.class);
        return (List<JSONObject>)response.getContent();
    }


    /**
     * 统计该楼层机动车位数量
     * @param context
     * @param request
     * @return
     */
    private Long countParkingCar(InstanceUrlParam context, CommonRequestDTO request){
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put(CommonConsant.CLASS_CODE, ClassCodeEnum.CFPKPS.getValue());
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria,RelationTypeEnum.MECHINARCH_EQ2FL.getGraphCode(),
                RelationTypeEnum.MECHINARCH_EQ2FL.getRelCode(), request.getFloorId(), 1);
        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JSONObject.class);
       return commonResult.getCount();
    }


    /**
     * 统计商铺数量
     * @param context
     * @param request
     * @return
     */
    private Long countStoreNum(InstanceUrlParam context, CommonRequestDTO request){
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put(CommonConsant.ROOM_LOCAL_FUNC_TYPE_ID, CommonConsant.STORE_BIMTYPEID);
        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria,RelationTypeEnum.ARCHSUBSET_FL2SP.getGraphCode(),
                RelationTypeEnum.ARCHSUBSET_FL2SP.getRelCode(), request.getFloorId(), 0);
        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JSONObject.class);
        return commonResult.getCount();
    }

}
