package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.middleware.client.DmpRwdClient;
import com.persagy.dmp.middleware.constant.CommonConsant;
import com.persagy.dmp.middleware.constant.enums.ClassCodeEnum;
import com.persagy.dmp.middleware.constant.enums.RelationTypeEnum;
import com.persagy.dmp.middleware.constant.enums.WdDefTypeEnum;
import com.persagy.dmp.middleware.constant.enums.WdModeCodeEnum;
import com.persagy.dmp.middleware.entity.dto.CommonRequestDTO;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.response.CommonResult;
import com.persagy.dmp.middleware.entity.response.DmpResult;
import com.persagy.dmp.middleware.entity.vo.WdDefineTypeRel;
import com.persagy.dmp.middleware.service.IBaseService;
import com.persagy.dmp.middleware.service.ISpaceService;
import com.persagy.dmp.middleware.service.IWdDefineTypeRelService;
import com.persagy.dmp.middleware.utils.EntityTransferUtil;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName ObjectInstanceServiceImpl
 * @Description: 广联达：查询空间信息
 * @Author linhuili
 * @Date 2021/9/10 11:17
 * @Version V1.0
 **/
@Service
public class SpaceServiceImpl implements ISpaceService {

    @Autowired
    private DmpRwdClient dmpRwdClient;

    @Autowired
    private IBaseService baseService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private IWdDefineTypeRelService iWdDefineTypeRelService;

    /**
     * 根据广联达空间类型查询空间信息
     * @param context
     * @param defineTypeRels
     * @param request
     */
    @Override
    public List<JSONObject> querySpaceByGldTypeRel(InstanceUrlParam context, List<WdDefineTypeRel> defineTypeRels, CommonRequestDTO request){
        if(CollUtil.isEmpty(defineTypeRels)){
            return new ArrayList<>();
        }
        //1. 筛选出bimTypeId为空间类型的数据：bimTypeId：FireZone
        List<WdDefineTypeRel> spaceTypeRels = filterSpaceType(context, defineTypeRels);

        //2. 根据bimTypeId查询楼层下的空间
        List<JSONObject> bimTypeSpace = querySpaceObjectByInfoCode(context, defineTypeRels, request, CommonConsant.ROOM_LOCAL_FUNC_TYPE_ID);
        //3. 数据组装：根据bimTypeId查询楼层下的空间结果
        List<JSONObject> bimTypeSpaceResult = combineQuerySpaceResultData(defineTypeRels, bimTypeSpace, CommonConsant.ROOM_LOCAL_FUNC_TYPE_ID);

        //4. 根据空间类型查询楼层下的空间
        List<JSONObject> classCodeSpace = querySpaceObjectByInfoCode(context, spaceTypeRels, request, CommonConsant.CLASS_CODE);
        //5. 数据组装：根据空间类型查询楼层下的空间结果
        List<JSONObject> classCodeSpaceResult = combineQuerySpaceResultData(spaceTypeRels, classCodeSpace,CommonConsant.CLASS_CODE);

        //6. 数据合并：根据bimTypeId查询的空间+根据空间类型查询到的空间
        List<JSONObject> result = new ArrayList<>();
        if(CollUtil.isNotEmpty(bimTypeSpaceResult)){
            result.addAll(bimTypeSpaceResult);
        }
        if(CollUtil.isNotEmpty(classCodeSpaceResult)){
            result.addAll(classCodeSpaceResult);
        }
        return result;
    }

    /**
     * 查询空间内的设备/阀门
     * @param context
     * @param request
     * @return
     */
    @Override
    public List<JSONObject> relQuery(InstanceUrlParam context, CommonRequestDTO request) throws Exception{
        //查询空间信息
        List<JSONObject> spaceList = querySpaceInfo(context, request);
        if(CollUtil.isEmpty(spaceList)){
            throw new Exception("空间不存在");
        }
        JSONObject space = spaceList.get(0);

        //获取空间的BIM构建编码
        String bimTypeId = space.getString(CommonConsant.ROOM_LOCAL_FUNC_TYPE_ID);

        //查询指定空间类型下的设备
        List<JSONObject> content = queryEquipInSpace(context, request, bimTypeId);
        return content;
    }


    /**
     * 查询指定空间下的设备
     * @param context
     * @param request
     * @param bimTypeId
     * @return
     * @throws Exception
     */
    @Override
    public List<JSONObject> queryEquipInSpace(InstanceUrlParam context, CommonRequestDTO request,String bimTypeId) throws Exception{
        //获取万达系统类下的设备类/阀门
        Set<String> wdSystemCodes = new HashSet<>();
        wdSystemCodes.add(request.getSystemCode());
        List<WdDefineTypeRel> equipByWdObjectType = getEquipByWdObjectType(wdSystemCodes);
        List<JSONObject> objects = new ArrayList<>();
        Set<String> bimTypeIds = new HashSet<>();
        if(CollUtil.isNotEmpty(equipByWdObjectType)){
            //查询空间下的设备/阀门
            bimTypeIds = equipByWdObjectType.stream().filter(k-> StringUtils.isNotEmpty(k.getBimTypeId())).map(k->k.getBimTypeId()).collect(Collectors.toSet());
            objects = queryDmpEquipInSpace(context, request, bimTypeIds);
        }

        //结果集封装
        return combineQuerySpaceResultData(equipByWdObjectType, objects, CommonConsant.BIM_TYPE_ID);
    }

    /**
     * 查询防火分区下的空间
     * @param context
     * @param request
     * @return
     */
    @Override
    public CommonResult queryFireZone(InstanceUrlParam context, CommonRequestDTO request) {
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();

        criteria.put(CommonConsant.CLASS_CODE, ClassCodeEnum.FIREZONE.getValue());
        QueryCriteria queryCriteria = new QueryCriteria();
        //根据建筑/楼层id关系查询
        if(StringUtils.isNotEmpty(request.getFloorId())){
            queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria, RelationTypeEnum.ARCHSUBSET_FL2SP.getGraphCode(),
                    RelationTypeEnum.ARCHSUBSET_FL2SP.getRelCode(), request.getFloorId(), 0);
        }else{
            queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria, RelationTypeEnum.ARCHSUBSET_BD2SP.getGraphCode(),
                    RelationTypeEnum.ARCHSUBSET_BD2SP.getRelCode(), request.getBuildId(), 0);
        }

        return baseService.doQuery(context, queryCriteria, JsonNode.class);
    }

    /**
     * 批量查询中台空间下的设备/阀门
     * @param context
     * @param request
     * @param bimTypeIds
     * @return
     */
    private List<JSONObject> queryDmpEquipInSpace(InstanceUrlParam context, CommonRequestDTO request,Set<String> bimTypeIds){
        List<JSONObject> result = new ArrayList<>();
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        ArrayNode arrayNode = criteria.withArray(CommonConsant.BIM_TYPE_ID);
        bimTypeIds.forEach(item->arrayNode.add(item));

        //处理空间id
        if(CollUtil.isEmpty(request.getSpaceIds())){
            Set<String> spaceId = new HashSet<>();
            spaceId.add(request.getSpaceId());
            request.setSpaceIds(spaceId);
        }

        //查询空间下的设备
        Set<String> spaceIds = request.getSpaceIds();
        for (String spaceId : spaceIds) {
            QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria,RelationTypeEnum.MECHINARCH_EQ2SP.getGraphCode(),
                    RelationTypeEnum.MECHINARCH_EQ2SP.getRelCode(),spaceId, 1);
            CommonResult response = baseService.doQuery(context,queryCriteria ,null);
            List<JSONObject> content = (List<JSONObject>) response.getContent();
            if(CollUtil.isNotEmpty(content)){
                result.addAll(content);
            }
        }
        return result;
    }

    /**
     * 根据万达对象类型查询设备/阀门信息
     * @param wdSystemCodes
     * @return
     */
    private List<WdDefineTypeRel> getEquipByWdObjectType(Set<String> wdSystemCodes){
        Set<String> objTypes = new HashSet<>();
        objTypes.add(WdDefTypeEnum.EQUIPMENT.getValue());
        objTypes.add(WdDefTypeEnum.MATERIAL.getValue());
        return iWdDefineTypeRelService.batchQuery(wdSystemCodes,objTypes);
    }


    /**
     * 组装查询空间结果
     * @param defineTypeRels
     * @param spaceList
     * @param infoCode
     * @return
     */
    private List<JSONObject>  combineQuerySpaceResultData(List<WdDefineTypeRel> defineTypeRels, List<JSONObject> spaceList, String infoCode){
        if(CollUtil.isEmpty(defineTypeRels)){
            return new ArrayList<>();
        }
        Map<String, List<JSONObject>> spaceMap = spaceList.stream().collect(Collectors.groupingBy(k->k.getString(infoCode)));
        //数据组装
        List<JSONObject> result = new ArrayList<>();
        for (WdDefineTypeRel defineTypeRel : defineTypeRels) {
            JSONObject data = new JSONObject();
            data.put("bimTypeId",defineTypeRel.getBimTypeId());
            data.put("className",defineTypeRel.getWdClassName());
            data.put("objType",defineTypeRel.getObjType());
            data.put("systemCode",defineTypeRel.getWdSystemCode());

            List<JSONObject> objects = new ArrayList<>();
            if(CollUtil.isNotEmpty(spaceMap)){
                objects = spaceMap.get(defineTypeRel.getBimTypeId());
            }
            data.put("objects",objects);
            data.put("count",objects == null ? 0 : objects.size());
            result.add(data);
        }
        return result;

    }

    /**
     * 根据信息点查询空间对象信息
     * @param context
     * @param defineTypeRels
     * @param request
     * @return
     */
    private List<JSONObject> querySpaceObjectByInfoCode(InstanceUrlParam context, List<WdDefineTypeRel> defineTypeRels, CommonRequestDTO request, String infoCode){
        //1. 根据bimTypeId查询楼层下的空间
        Set<String> bimTypeIdSets = defineTypeRels.stream().filter(k-> StringUtils.isNotEmpty(k.getBimTypeId())).map(k->k.getBimTypeId()).collect(Collectors.toSet());
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        ArrayNode bimTypeIds = criteria.withArray(infoCode);
        bimTypeIdSets.forEach(item->{bimTypeIds.add(item);});

        QueryCriteria queryCriteria = EntityTransferUtil.transDmpRelationCriteria(criteria, RelationTypeEnum.ARCHSUBSET_FL2SP.getGraphCode(),
                RelationTypeEnum.ARCHSUBSET_FL2SP.getRelCode(), request.getFloorId(), 0);
        CommonResult commonResult = baseService.doQuery(context, queryCriteria, JsonNode.class);
        return (List<JSONObject>)commonResult.getContent();
    }


    /**
     * 获取所有的空间类型
     * @param instanceUrlParam
     * @return
     * @throws IOException
     */
    private List<JSONObject> getDmpBaseSpaceType(InstanceUrlParam instanceUrlParam){
        //封装查询条件
        QueryCriteria queryCriteria = new QueryCriteria();
        ObjectNode criteria = JsonNodeFactory.instance.objectNode();
        criteria.put("objType","space");
        queryCriteria.setCriteria(criteria);

        //查询空间类型
        JSONObject object = null;
        try {
            String jsonStr = objectMapper.writeValueAsString(queryCriteria);
            object = JSONObject.parseObject(jsonStr);
        } catch (IOException e) {
            e.printStackTrace();
        }
        DmpResult<JSONArray> typeResult = dmpRwdClient.queryObjectType(instanceUrlParam, object);
        return EntityTransferUtil.toMultiEntity(typeResult.getData());
    }


    /**
     * 筛选出bimTypeId为空间类型的数据：bimTypeId：FireZone
     * @param instanceUrlParam
     * @param defineTypeRels
     * @return
     */
    private List<WdDefineTypeRel> filterSpaceType(InstanceUrlParam instanceUrlParam, List<WdDefineTypeRel> defineTypeRels){
        if(CollUtil.isEmpty(defineTypeRels)){
            return new ArrayList<>();
        }
        List<WdDefineTypeRel> spaceTypeRels = new ArrayList<>();
        //获取中台基础空间类型
        List<JSONObject> dmpSpaceTypeList = getDmpBaseSpaceType(instanceUrlParam);
        Set<String> dmpSpaceTypeCodes= new HashSet<>();
        if(CollUtil.isNotEmpty(dmpSpaceTypeList)){
           dmpSpaceTypeCodes = dmpSpaceTypeList.stream().map(k->k.getString("code")).collect(Collectors.toSet());
        }

        //移除非bimTypeID格式的数据，并单独存储
        Iterator<WdDefineTypeRel> iterator = defineTypeRels.iterator();
        while (iterator.hasNext()){
            WdDefineTypeRel wdDefineTypeRel = iterator.next();
            if(dmpSpaceTypeCodes.contains(wdDefineTypeRel.getBimTypeId())){
                spaceTypeRels.add(wdDefineTypeRel);
                iterator.remove();
            }
        }
        return spaceTypeRels;
    }


    /**
     * 查询空间信息
     * @param context
     * @param requestDTO
     * @return
     */
    private List<JSONObject> querySpaceInfo(InstanceUrlParam context, CommonRequestDTO requestDTO){
        ObjectNode jsonNodes = JsonNodeFactory.instance.objectNode();
        jsonNodes.put("id",requestDTO.getSpaceId());
        QueryCriteria criteria = new QueryCriteria();
        criteria.setCriteria(jsonNodes);

        CommonResult response = baseService.doQuery(context, criteria, JSONObject.class);
        return (List<JSONObject>)response.getContent();
    }

}
