package com.persagy.dmp.middleware.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.dmp.middleware.dao.WdDefineTypeRelMapper;
import com.persagy.dmp.middleware.entity.dto.InstanceUrlParam;
import com.persagy.dmp.middleware.entity.vo.WdDefineTypeRel;
import com.persagy.dmp.middleware.service.IWdDefineTypeRelService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @ClassName ObjectInstanceServiceImpl
 * @Description: 广联达：关系映射表配置
 * @Author linhuili
 * @Date 2021/9/10 11:17
 * @Version V1.0
 **/
@Service
public class WdDefineTypeRelServiceImpl implements IWdDefineTypeRelService {

    @Autowired
    private WdDefineTypeRelMapper wdDefineTypeRelMapper;

    /**
     * 查询广联达映射表
     * @param context
     * @param wdDefineTypeRel
     * @return
     */
    @Override
    public List<WdDefineTypeRel> query(InstanceUrlParam context, WdDefineTypeRel wdDefineTypeRel) {
        QueryWrapper<WdDefineTypeRel> wrapper = new QueryWrapper<>();
        if(StringUtils.isNotEmpty(wdDefineTypeRel.getObjType())){
            wrapper.eq("obj_type",wdDefineTypeRel.getObjType());
        }
        if(StringUtils.isNotEmpty(wdDefineTypeRel.getBimTypeId())){
            wrapper.eq("bim_type_id",wdDefineTypeRel.getBimTypeId());
        }
        if(StringUtils.isNotEmpty(wdDefineTypeRel.getWdSystemCode())){
            wrapper.eq("wd_system_code",wdDefineTypeRel.getWdSystemCode());
        }
        if(StringUtils.isNotEmpty(wdDefineTypeRel.getWdModelCode())){
            wrapper.eq("wd_model_code",wdDefineTypeRel.getWdModelCode());
        }
        wrapper.isNotNull("wd_class_name");
        //查询广联达映射表
        List<WdDefineTypeRel> defineTypeRels = wdDefineTypeRelMapper.selectList(wrapper);
        //去除重复数据，相同类型名称，相同bimTypeID
        removeDuplicateData(defineTypeRels);
        return defineTypeRels;
    }


    /**
     * 批量查询映射表
     * @param wdSystemCodes
     * @param objTypes
     * @return
     */
    @Override
    public List<WdDefineTypeRel> batchQuery(Set<String> wdSystemCodes, Set<String> objTypes) {
        QueryWrapper<WdDefineTypeRel> wrapper = new QueryWrapper<>();
        wrapper.in("obj_type",objTypes);
        wrapper.in("wd_system_code",wdSystemCodes);
        List<WdDefineTypeRel> defineTypeRels = wdDefineTypeRelMapper.selectList(wrapper);
        //去除重复数据
        removeDuplicateData(defineTypeRels);
        return defineTypeRels;
    }

    /**
     * 去除重复数据
     * @param lists
     */
    public static void removeDuplicateData(List<WdDefineTypeRel> lists){
        Set<String> keys = new HashSet<>();
        Iterator<WdDefineTypeRel> iterator = lists.iterator();
        while (iterator.hasNext()){
            WdDefineTypeRel wdDefineTypeRel = iterator.next();
            if(StringUtils.isEmpty(wdDefineTypeRel.getWdClassName())){
                iterator.remove();
                continue;
            }
            if(CollUtil.isNotEmpty(keys) && keys.contains(wdDefineTypeRel.getWdClassName()+"-"+wdDefineTypeRel.getBimTypeId())){
                iterator.remove();
                continue;
            }
            keys.add(wdDefineTypeRel.getWdClassName()+"-"+wdDefineTypeRel.getBimTypeId());
        }
    }

}
