package com.persagy.dmp.middleware.utils;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;

import java.text.CollationKey;
import java.text.Collator;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 排序
 */
public class SortUtil {

    /**
     * 按首字母分组排序，并且数量为0的在后面
     * @param list
     * @return
     */
    public static List<JSONObject> sortByCharacter(List<JSONObject> list, String groupType,String sortName) throws Exception {
        if (CollUtil.isEmpty(list)) {
            return new ArrayList<>();
        }

        //按对象类型分组处理
        Map<String, List<JSONObject>> groupMap = list.stream().collect(Collectors.groupingBy(k-> k.getString(groupType),LinkedHashMap::new,Collectors.toList()));

        //按首字母，按objType，按为0的
        List<JSONObject> result = new ArrayList<>();
        List<JSONObject> zoreList = new ArrayList<>();
        for(Map.Entry<String, List<JSONObject>> entry : groupMap.entrySet()){
            //按中文首字母排序
            List<JSONObject> objects = entry.getValue();
            sortByAlphabetical(objects,sortName);

            //去重为0的数据,单独存储
            Iterator<JSONObject> iterator = objects.iterator();
            while (iterator.hasNext()){
                JSONObject next = iterator.next();
                if(next.getInteger("count") == null || next.getInteger("count") == 0){
                    zoreList.add(next);
                    iterator.remove();
                }
            }
            result.addAll(objects);
        }
        if(CollUtil.isNotEmpty(zoreList)){
            result.addAll(zoreList);
        }
        return result;
    }

    /**
     * 按首字母排序
     * @param list
     * @param name
     */
    public static void sortByAlphabetical(List<JSONObject> list, String name) {
        Collections.sort(list, new Comparator<JSONObject>() {
            Collator collator = Collator.getInstance(Locale.CHINA);
            @Override
            public int compare(JSONObject o1, JSONObject o2) {
                CollationKey key1 = collator.getCollationKey(o1.getString(name));
                CollationKey key2 = collator.getCollationKey(o2.getString(name));
                return key1.compareTo(key2);
            }
        });
    }


}
